package ec.app.sequence;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.commons.io.FileUtils;

public class RNASequenceParser {

	public static void generateSequences(String chromosomeId,
			String actualSequence, ArrayList donorList, ArrayList acceptorList,
			ArrayList positiveAcceptorList, ArrayList positiveDonorList) {
		for (int i = 0; i < actualSequence.length(); i++) {
			int endIndex = i + 162;
			if (endIndex < actualSequence.length()) {
				// get the substring
				String subSequence = actualSequence.substring(i, endIndex);				
				String acceptorLabel = "-1";	
				if(positiveAcceptorList.contains(subSequence)){
					acceptorLabel = "1";
				}
				String donorLabel = "-1";	
				if(positiveDonorList.contains(subSequence)){
					donorLabel = "1";
				}
				// add it to both donor and acceptor
				String totalAcceptorSequence = subSequence + " " + acceptorLabel + " " + chromosomeId;
				// add it to both donor and acceptor
				String totalDonorSequence = subSequence + " " + donorLabel + " " + chromosomeId;
				donorList.add(totalDonorSequence);
				acceptorList.add(totalAcceptorSequence);
			}

		}
	}

	private static void readSequenceToList(String file, List dataList)
			throws Exception {
		// read the sequences
		BufferedReader reader = new BufferedReader(new InputStreamReader(
				new FileInputStream(file)));
		// read the first line
		String totalLine = reader.readLine();
		while (totalLine != null) {
			int startOfSpace = totalLine.indexOf(" ");
			String sequence = totalLine.substring(0, startOfSpace);
			dataList.add(sequence);
			totalLine = reader.readLine();
		}
	}

	public static void main(String[] args) throws Exception {
		if (args.length < 5) {
			System.err
					.println("RNASequenceParser dataFile donorSaveFile accecptorSaveFile positiveDonorFile positiveAcceptorFile id");
			System.exit(-1);
		}
		String id = args[5];
		String positiveAcceptorFile = args[4];
		String positiveDonorFile = args[3];

		// read the postive acceptors
		ArrayList positiveAcceptors = new ArrayList();
		readSequenceToList(positiveAcceptorFile, positiveAcceptors);

		// read the positive donors
		ArrayList positiveDonors = new ArrayList();
		readSequenceToList(positiveDonorFile, positiveDonors);

		// read the sequences
		BufferedReader reader = new BufferedReader(new InputStreamReader(
				new FileInputStream(args[0])));
		// read the first line
		String totalLine = reader.readLine();
		ArrayList donorList = new ArrayList();
		ArrayList acceptorList = new ArrayList();
		while (totalLine != null) {
			int startOfSpace = totalLine.indexOf(" ");
			String chromosomeId = totalLine.substring(0, startOfSpace);
			String sequence = totalLine.substring(startOfSpace + 1, totalLine
					.length());
			if (chromosomeId.equals(id)) {
				// now let the parser understand what to do
				generateSequences(chromosomeId, sequence, donorList,
						acceptorList, positiveAcceptors, positiveDonors);
				break;
			}
			totalLine = reader.readLine();
		}
		// print the donorsamples[2]
		FileUtils.writeLines(new File(args[1]), donorList);
		FileUtils.writeLines(new File(args[2]), acceptorList);
	}

}
