package ec.app.sequence.func;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

import ec.EvolutionState;
import ec.Problem;
import ec.app.sequence.MotifData;
import ec.gp.ADFStack;
import ec.gp.ERC;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Code;
import ec.util.DecodeReturn;
import ec.util.Parameter;
/**
 * This is an Integer based ERC, for generating Positions. It will generate Uniform Random Integers
 * between maxPosition and minPosition. If maxPosition = 140, minPosition =0, it will generate Positional
 * Features. If maxPosition = 140, minPosition = 60, it will generate only in a downstream of [-60,+80]
 * sequence, simillarly if maxPosition =60, minPosition =0, it will generate in upstream.
 * 
 * @author udaykamath
 *
 */
public class IntegerTerminal extends ERC{
	int positionMax;
	int positionMin;
	int x;
	public void resetNode(final EvolutionState state, final int thread) {
		int range = positionMax - positionMin + 1;
		long fraction = (long)(range * state.random[thread].nextDouble());
	    x =  (int)(fraction + positionMin); 		
	}
	
	public void setup(final EvolutionState state, final Parameter base)
    {
		super.setup(state, base);
		//check for position lengths
		positionMax = state.parameters.getInt(base.push("maxPosition"),
	            null);
		positionMin = state.parameters.getInt(base.push("minPosition"),
	            null);
		if (positionMax == 0)
			positionMax = 162;
		
    }
	
	public String name() {
		return "'" + new Integer(x).toString() + "'";
	}
	public String toString() {
		return " @ " + new Integer(x).toString() + "";
	}

	public int nodeHashCode() {
		return this.getClass().hashCode() + Integer.bitCount(x);
	}

	public boolean nodeEquals(final GPNode node) {
		if (this.getClass() != node.getClass())
			return false;
		IntegerTerminal node1 = (IntegerTerminal) node;
		if (node1.x == this.x)
			return true;
		return false;
	}

	public void readNode(final EvolutionState state, final DataInput dataInput)
			throws IOException {
		x = dataInput.readInt();
	}

	public void writeNode(final EvolutionState state,
			final DataOutput dataOutput) throws IOException {
		dataOutput.writeInt(x);
	}

	public String encode() {
		return Code.encode(x);
	}

	public boolean decode(DecodeReturn dret) {
		// store the position and the string in case they
		// get modified by Code.java
		int pos = dret.pos;
		String data = dret.data;

		// decode
		Code.decode(dret);

		if (dret.type != DecodeReturn.T_INT) // uh oh!
		{
			// restore the position and the string; it was an error
			dret.data = data;
			dret.pos = pos;
			return false;
		}

		// store the data
		x = new Integer(data).intValue();

		return true;
	}


    public void checkConstraints(final EvolutionState state,
        final int tree,
        final GPIndividual typicalIndividual,
        final Parameter individualBase)
        {
        super.checkConstraints(state,tree,typicalIndividual,individualBase);
        if (children.length!=0)
            state.output.error("Incorrect number of children for node " + 
                toStringForError() + " at " +
                individualBase);
        }

    public void eval(final EvolutionState state,
        final int thread,
        final GPData input,
        final ADFStack stack,
        final GPIndividual individual,
        final Problem problem)
        {
    	MotifData motifData = ((MotifData) (input));
		
		motifData.currentPosition = x;
		
        }
    
    public int getRandom(final EvolutionState state, final int thread){
    	int range = positionMax - positionMin + 1;
		long fraction = (long)(range * state.random[thread].nextDouble());
	    x =  (int)(fraction + positionMin);
	    return x;
	}
    
}

