package ec.app.sequence.func;

import ec.EvolutionState;
import ec.Problem;
import ec.app.sequence.MotifData;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;
/**
 * This function or non-terminal is used to do Not/! connection between two Nodes, can
 * be Compositional, Regional, Correlational etc. This is used for non-presence of certain 
 * Motif which has higher Information Gain and discriminating power.
 * @author udaykamath
 *
 */
public class Not extends GPNode {
	public String toString() {
		return " Not ";
	}

	public void checkConstraints(final EvolutionState state, final int tree,
			final GPIndividual typicalIndividual, final Parameter individualBase) {
		super.checkConstraints(state, tree, typicalIndividual, individualBase);
		if (children.length != 1)
			state.output.error("Incorrect number of children for node "
					+ toStringForError() + " at " + individualBase);
	}

	public void eval(final EvolutionState state, final int thread,
			final GPData input, final ADFStack stack,
			final GPIndividual individual, final Problem problem) {
		boolean result;
		MotifData motifData = ((MotifData) (input));

		children[0].eval(state, thread, input, stack, individual, problem);
		result = motifData.matches;
		//invert it
		motifData.matches = !(result);
	}

}
