package ec.app.sequence.func;

import org.biojava.bio.symbol.SymbolList;
import org.biojava.utils.regex.Matcher;
import org.biojava.utils.regex.Pattern;
import org.biojava.utils.regex.PatternFactory;

import ec.EvolutionState;
import ec.Problem;
import ec.app.sequence.MotifData;
import ec.app.sequence.SequenceClassificationProblem;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;
/**
 * This function or non-terminal is used for generating "Region" or "Positional" Features.
 * It takes two parameters Motif and Position. If Position is any uniformly generated integer
 * it will find positional features, if position is restricted to Upstream or Downstream and controlled
 * to be further in certain areas it will specialize as Regional Feature. 
 * @author udaykamath
 *
 */
public class PositionBased extends GPNode{
	
	public String toString() {
		return " matchesAtPosition ";
	}

	public void checkConstraints(final EvolutionState state, final int tree,
			final GPIndividual typicalIndividual, final Parameter individualBase) {
		super.checkConstraints(state, tree, typicalIndividual, individualBase);
		if (children.length != 2)
			state.output.error("Incorrect number of children for node "
					+ toStringForError() + " at " + individualBase);
	}

	public void eval(final EvolutionState state, final int thread,
			final GPData input, final ADFStack stack,
			final GPIndividual individual, final Problem problem) {
		MotifData motifData = ((MotifData) (input));
		//child should give me motif
		children[0].eval(state, thread, input, stack, individual, problem);	
		//child should give me motif
		children[1].eval(state, thread, input, stack, individual, problem);		
		//get the curernt motif
		char[] currentMotif = motifData.currentMotif;
		//get the current position
		int position = motifData.currentPosition;
		//check for match using biojava
		SequenceClassificationProblem spliceSetProblem =(SequenceClassificationProblem)problem;
		SymbolList sequence = spliceSetProblem.getCurrentSequence();
		PatternFactory factory = spliceSetProblem.getFactory(thread);
		motifData.matches = matches(sequence,currentMotif,position, factory);
	}
	
	private boolean matches(SymbolList seq, char[] currentMotif, int position,PatternFactory factory){
		boolean matched = false;
		try{
			//in the sequence find the match
			Pattern p = factory.compile(new String(currentMotif));
//			Pattern p = PromoterSetProblem.factory.compile(new String(currentMotif));
			Matcher occurences= p.matcher(seq);
		    while(occurences.find()){
		    	if(occurences.start() == position){
		    		matched = true;
		    		break;
		    	}
		    }
		}catch(Exception e){
			System.out.println("Exception at Motif: " + new String(currentMotif) + " position:" + position + " seq:" + seq.seqString());
			e.printStackTrace();
		}
		return matched;
	}
}
